@@ -1,4 +1,3 @@
-from warnings import warn
 from ewokscore.cliutils import add_execute_parameters as _add_execute_parameters
 from ewokscore.cliutils import apply_execute_parameters
 from ewokscore.cliutils import add_convert_parameters  # noqa F401
@@ -28,8 +27,3 @@ def add_submit_parameters(parser, shell=False):
 
 def apply_submit_parameters(args, shell=False):
     apply_execute_parameters(args, shell=shell)
-    if args.binding:
-        if args.engine:
-            raise ValueError("--binding and --engine cannot be used together")
-        args.engine = args.binding
-        warn("--binding is deprecated in favor of --engine", FutureWarning)
